@@ -2,9 +2,9 @@
 # Mail::SPF::Mech::PTR
 # SPF record "ptr" mechanism class.
 #
-# (C) 2005-2006 Julian Mehnle <julian@mehnle.net>
+# (C) 2005-2012 Julian Mehnle <julian@mehnle.net>
 #     2005      Shevek <cpan@anarres.org>
-# $Id: PTR.pm 22 2006-11-15 03:31:28Z Julian Mehnle $
+# $Id: PTR.pm 57 2012-01-30 08:15:31Z julian $
 #
 ##############################################################################
 
@@ -19,12 +19,10 @@ Mail::SPF::Mech::PTR - SPF record C<ptr> mechanism class
 use warnings;
 use strict;
 
-use base 'Mail::SPF::Mech';
+use base 'Mail::SPF::SenderIPAddrMech';
 
 use Mail::SPF::Util;
 
-use NetAddr::IP;
-
 use constant TRUE   => (0 == 0);
 use constant FALSE  => not TRUE;
 
@@ -59,10 +57,11 @@ See L<Mail::SPF::Mech/new>.
 
 =back
 
-=item B<new_from_string($text)>: returns I<Mail::SPF::Mech::PTR>;
+=item B<new_from_string($text, %options)>: returns I<Mail::SPF::Mech::PTR>;
 throws I<Mail::SPF::ENothingToParse>, I<Mail::SPF::EInvalidMech>
 
-Creates a new SPF record C<ptr> mechanism object by parsing the given string.
+Creates a new SPF record C<ptr> mechanism object by parsing the string and
+any options given.
 
 =back
 
@@ -144,7 +143,7 @@ sub match {
     $server->count_dns_interactive_term($request);
     return
         Mail::SPF::Util->valid_domain_for_ip_address(
-            $server, $request->ip_address, $self->domain($server, $request))
+            $server, $request, $request->ip_address, $self->domain($server, $request))
         ? TRUE : FALSE;
 }
 
@@ -154,7 +153,7 @@ sub match {
 
 L<Mail::SPF>, L<Mail::SPF::Record>, L<Mail::SPF::Term>, L<Mail::SPF::Mech>
 
-L<RFC 4408|http://www.ietf.org/rfc/rfc4408.txt>
+L<http://tools.ietf.org/html/rfc4408>
 
 For availability, support, and license information, see the README file
 included with Mail::SPF.
